package com.gic.haoban.manage.web.controller;

import java.io.IOException;
import java.util.*;
import java.util.stream.Collectors;

import javax.servlet.http.HttpServletResponse;

import com.gic.haoban.app.customer.service.api.service.QywxTagSyncApiService;
import com.google.common.collect.Lists;
import com.google.common.collect.Sets;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import cn.hutool.core.collection.CollectionUtil;

import com.gic.api.base.commons.Page;
import com.gic.commons.util.GlobalVar;
import com.gic.enterprise.api.dto.EnterpriseDTO;
import com.gic.enterprise.api.dto.StoreDTO;
import com.gic.enterprise.api.service.EnterpriseService;
import com.gic.enterprise.api.service.StoreService;
import com.gic.haoban.base.api.common.BasePageInfo;
import com.gic.haoban.base.api.common.PageResult;
import com.gic.haoban.common.utils.EntityUtil;
import com.gic.haoban.common.utils.HaobanResponse;
import com.gic.haoban.common.utils.UuidUtil;
import com.gic.haoban.manage.api.dto.DepartmentDTO;
import com.gic.haoban.manage.api.dto.EnterpriseDetailDTO;
import com.gic.haoban.manage.api.dto.YwWxEnterpriseDTO;
import com.gic.haoban.manage.api.service.DepartmentApiService;
import com.gic.haoban.manage.api.service.WxEnterpriseApiService;
import com.gic.haoban.manage.api.service.WxEnterpriseRelatedApiService;
import com.gic.haoban.manage.web.errCode.HaoBanErrCode;
import com.gic.haoban.manage.web.vo.EnterpriseDetailVo;
import com.gic.redis.data.util.RedisUtil;

@RestController
public class EnterpriseController extends WebBaseController {
    private static final String LOGIN_URL = "/haoban-manage3-web/yw-login";
    private static Logger logger = LoggerFactory.getLogger(EnterpriseController.class);

    @Autowired
    private WxEnterpriseApiService wxEnterpriseApiService;
    @Autowired
    private WxEnterpriseRelatedApiService wxEnterpriseRelatedApiService;
    @Autowired
    private DepartmentApiService departmentApiService;
    @Autowired
    private StoreService storeService;
    @Autowired
    private EnterpriseService enterpriseService;
    @Autowired
    private QywxTagSyncApiService qywxTagSyncApiService;

    //	@Autowired
//	private WxEnterpriseRelatedApiService wxEnterpriseRelllatedApiService;
    //企业列表
    // openTagFlag 0 未开启 1 开启 2 所有
    @RequestMapping("wx-enterprise-list")
    public HaobanResponse wxEnterpriseList(BasePageInfo pageInfo, String keyword, Integer openTagFlag, String gicEnterpriseName, Integer gicContactHelper, String version) {
        List<String> wxEnterpriseIds = new ArrayList<String>();
        if (!StringUtils.isEmpty(gicEnterpriseName)) {
            List<EnterpriseDTO> enterpriseNames = enterpriseService.getEnterpriseByName(gicEnterpriseName);
            if (CollectionUtil.isNotEmpty(enterpriseNames)) {
                List<String> enterpriseIds = enterpriseNames.stream().map(s -> s.getEnterpriseId()).collect(Collectors.toList());
                List<EnterpriseDetailDTO> wxEnterpriseIdDTOs = wxEnterpriseRelatedApiService.listEnterpriseByEnterpriseIds(enterpriseIds);
                if (CollectionUtil.isNotEmpty(wxEnterpriseIdDTOs)) {
                    wxEnterpriseIds = wxEnterpriseIdDTOs.stream().map(s -> s.getWxEnterpriseId()).collect(Collectors.toList());
                }
            }
            if (CollectionUtil.isEmpty(wxEnterpriseIds)) {
                wxEnterpriseIds = Arrays.asList("-9999");
            }
        }

        // 查询开启标签同步缓存-详情在 /tag/open-tag 接口
        String wxEntIdStr = (String) RedisUtil.getCache("qywx-tag-open");
        Set<String> wxEntIdSet = Sets.newHashSet();

        if (StringUtils.isNotEmpty(wxEntIdStr)) {
            wxEntIdSet = Arrays.stream(wxEntIdStr.split(",")).collect(Collectors.toSet());
        }

        Page<YwWxEnterpriseDTO> page = wxEnterpriseApiService.list(pageInfo, keyword, wxEnterpriseIds, gicContactHelper, version);
        PageResult<YwWxEnterpriseDTO> pageVo = new PageResult<>();
        List<YwWxEnterpriseDTO> list = EntityUtil.changeEntityListByJSON(YwWxEnterpriseDTO.class, page.getResult());

        List<YwWxEnterpriseDTO> resList = Lists.newArrayListWithCapacity(list.size());

        for (YwWxEnterpriseDTO dto : list) {
            dto.setVersion("3.0");
            dto.setBuyDate(new Date());
            dto.setBuyStatus("1");
            List<EnterpriseDetailDTO> counts = wxEnterpriseRelatedApiService.listEnterpriseByWxEnterpriseId(dto.getWxEnterpriseId());
            dto.setRelationCount(counts.size());
            if (openTagFlag == 0 && !wxEntIdSet.contains(dto.getWxEnterpriseId())) {
                dto.setTagOpenFlag(0);
                resList.add(dto);
            } else if (openTagFlag == 1 && wxEntIdSet.contains(dto.getWxEnterpriseId())) {
                dto.setTagOpenFlag(1);
                resList.add(dto);
            } else if (openTagFlag == 2) {
                dto.setTagOpenFlag(wxEntIdSet.contains(dto.getWxEnterpriseId()) ? 1 : 0);
                resList.add(dto);
            }
        }

        pageVo.setList(resList);
        pageVo.setPageNum(page.getCurrentPage());
        pageVo.setPages(page.getPages());
        // 因为过滤了此处取list长度
        pageVo.setPageSize(page.getPageSize());
        pageVo.setTotal(page.getTotalCount());
        return resultResponse(HaoBanErrCode.ERR_1, pageVo);

    }

    /**
     * 开启关闭标签同步
     *
     * @param wxEnterpriseId 微信企业id
     * @param flag           操作标记  1 开启 0 关闭
     * @return HaobanResponse
     */
    @RequestMapping("operate-tag-sync")
    public HaobanResponse operateTagSync(String wxEnterpriseId, Integer flag) {
        if (StringUtils.isEmpty(wxEnterpriseId)) {
            return resultResponse(HaoBanErrCode.ERR_2);
        }
        if (Objects.isNull(flag)) {
            return resultResponse(HaoBanErrCode.ERR_2);
        }
        // 操作缓存
        String key = "qywx-tag-open";
        String lockKey = "qywx-tag-open-lock";
        try {
            RedisUtil.lock(lockKey, 5L);

            String wxEntIdStr = (String) RedisUtil.getCache(key);

            if (flag == 1) {
                // 开启
                if (StringUtils.isEmpty(wxEntIdStr)) {
                    wxEntIdStr = wxEnterpriseId;
                } else {
                    wxEntIdStr =  wxEntIdStr + "," + wxEnterpriseId;
                }
                logger.info("开启企业标签同步, wxEntIdStr:{}", wxEntIdStr);
                RedisUtil.setCache(key, wxEntIdStr);
            }

            if (flag == 0) {
                // 关闭标签检查是否有任务在进行
                boolean tagSync = qywxTagSyncApiService.checkQywxTagSync(wxEnterpriseId);
                if (!tagSync) {
                    // 有同步任务
                    return resultResponse(HaoBanErrCode.ERR_30001);
                }
            }

            if (flag == 0 && StringUtils.isNotEmpty(wxEntIdStr)) {
                // 关闭标签同步，只有在字符串不为空时才能关
                List<String> filterList = Arrays.stream(wxEntIdStr.split(","))
                        .filter(one -> !one.equals(wxEnterpriseId))
                        .collect(Collectors.toList());
                String closeStr = StringUtils.join(filterList, ",");
                logger.info("关闭企业标签同步, closeStr:{}", closeStr);
                RedisUtil.setCache(key, closeStr);
            }

        } finally {
            RedisUtil.unlock(lockKey);
        }

        // 文案提示-根据开启关闭类型提示
        if (flag == 1) {
            return resultResponse(HaoBanErrCode.ERR_1, "开启企微标签同步");
        }
        if(flag == 0) {
            return resultResponse(HaoBanErrCode.ERR_1, "关闭企微标签同步");
        }

        return resultResponse(HaoBanErrCode.ERR_1);
    }

    //授权详情列表
    @RequestMapping("authorize-detail")
    public HaobanResponse authorizeDetail(String wxEnterpriseId) {
        List<EnterpriseDetailDTO> list = wxEnterpriseRelatedApiService.listEnterpriseByWxEnterpriseId(wxEnterpriseId);
        List<EnterpriseDetailVo> result = EntityUtil.changeEntityListByJSON(EnterpriseDetailVo.class, list);
        for (EnterpriseDetailVo vo : result) {
            List<StoreDTO> stores = storeService.getStoreByEnterId(vo.getEnterpriseId(), null, null);
            if (CollectionUtil.isEmpty(stores)) {
                continue;
            } else {
                List<String> storeIds = stores.stream().map(s -> s.getStoreId()).collect(Collectors.toList());
                List<DepartmentDTO> countList = departmentApiService.listByRelatedIds(storeIds);
                vo.setStoreCount(countList.size());
            }
            EnterpriseDTO enterpriseDTO = enterpriseService.getEnterpriseById(vo.getEnterpriseId());
            vo.setEnterpriseName(enterpriseDTO.getEnterpriseName());
            vo.setEnterpriseLogo(enterpriseDTO.getQcloudImageUrl());
        }
        return resultResponse(HaoBanErrCode.ERR_1, result);
    }

    //登录
    @RequestMapping("yw-login")
    public void ywLogin(String wxEnterpriseId, HttpServletResponse httpServletResponse) throws IOException {
        String host = GlobalVar.ctxPropertiesMap.get("haoban_service_host");
        logger.info("host============================>{}", host);
        String random = "ywLogin:" + UuidUtil.randomUUID();
        RedisUtil.setCache(random, 1, 1000L);
        String url = host + LOGIN_URL + "?wxEnterpriseId=" + wxEnterpriseId + "&random=" + random;
        httpServletResponse.sendRedirect(url);
    }
}
